#define G_LOG_DOMAIN "gpod-library-item"

#include "config.h"

#include "gpod-library-item.h"

#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <glib/gi18n.h>

/**
 * SECTION:gpod-library-view
 * @title: GpodLibraryItem
 * @short_description: A #GtkBin widget for showing a feed in the #GpodLibraryView
 * @include: gpod-library-item.h
 * @see_also: #GpodLibraryView
 *
 *  A #GtkBin widget for showing a feed in the #GpodLibraryView
 **/

typedef struct _GpodLibraryItemPrivate
{
    GtkWidget *box;
    gchar     *url;
    GtkImage  *image;
    gchar     *image_url;
    GtkLabel  *title;
} GpodLibraryItemPrivate;

struct _GpodLibraryItem
{
    GtkBox parent;
};

G_DEFINE_TYPE_WITH_PRIVATE (GpodLibraryItem, gpod_library_item, GTK_TYPE_BIN)

enum {
    PROP_0,
    PROP_URL,
    PROP_TITLE,
    PROP_IMAGE,
    LAST_PROP
};

static GParamSpec *properties[LAST_PROP] = { NULL, };

/*
static void
on_image_load (GpodDownloader *downloader,
               gchar          *file,
               gpointer        user_data)
{
    g_return_if_fail (GPOD_IS_DOWNLOADER (downloader));
    g_return_if_fail (GPOD_IS_LIBRARY_ITEM (user_data));

    //GpodLibraryItem *self = GPOD_LIBRARY_ITEM (user_data);
    //GpodLibraryItemPrivate *priv = gpod_library_item_get_instance_private (self);

    //gtk_image_set_from_file (priv->image, file);
}

static gboolean
load_image_async (gpointer user_data)
{
    g_return_val_if_fail (GPOD_IS_LIBRARY_ITEM (user_data), FALSE);

    GpodLibraryItem *self = GPOD_LIBRARY_ITEM (user_data);
    //GpodLibraryItemPrivate *priv = gpod_library_item_get_instance_private (self);
    GpodDownloader *downloader = gpod_downloader_new ();


    g_signal_connect (downloader,
                      "downloaded",
                      G_CALLBACK (on_image_load),
                      self);

    return FALSE;
}
*/

static void
gpod_library_item_set_property (GObject      *object,
                                guint         prop_id,
                                const GValue *value,
                                GParamSpec   *pspec)
{
    GpodLibraryItem *self = GPOD_LIBRARY_ITEM (object);
    GpodLibraryItemPrivate *priv = gpod_library_item_get_instance_private (self);

    switch (prop_id)
    {
        case PROP_URL:
            priv->url = g_value_dup_string (value);
            break;

        case PROP_TITLE:
            gtk_label_set_label (priv->title, g_value_get_string (value));
            break;

        case PROP_IMAGE:
            priv->image_url = g_value_dup_string (value);
            // TODO: segfault
            //g_idle_add (load_image_async, self);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    }
}

static void
gpod_library_item_get_property (GObject    *object,
                                guint       prop_id,
                                GValue     *value,
                                GParamSpec *pspec)
{
    GpodLibraryItem *self = GPOD_LIBRARY_ITEM (object);
    GpodLibraryItemPrivate *priv = gpod_library_item_get_instance_private (self);

    switch (prop_id)
    {
        case PROP_URL:
            g_value_set_string (value, priv->url);
            break;

        case PROP_TITLE:
            g_value_set_string (value, gtk_label_get_text (priv->title));
            break;

        case PROP_IMAGE:
            //g_value_set_string (value, gtk_entry_get_text (priv->entry));
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    }
}

/*
 * gpod_library_item_finalize:
 * @object: An instance of #GpodLibraryItem
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
gpod_library_item_finalize (GObject *object)
{
    GpodLibraryItem *self = GPOD_LIBRARY_ITEM (object);
    GpodLibraryItemPrivate* priv = gpod_library_item_get_instance_private (self);

    g_clear_pointer (&priv->url, g_free);

    G_OBJECT_CLASS (gpod_library_item_parent_class)->finalize(object);
}

/*
 * gpod_library_item_init:
 * @self: An instance of #GpodLibraryItem
 *
 * Initializes various aspects of the #GpodLibraryItem. Anything the #GpodLibraryItem
 * would need before becoming presented should be done here.
 */
static void
gpod_library_item_init (GpodLibraryItem *self)
{
    GpodLibraryItemPrivate *priv = gpod_library_item_get_instance_private (self);

    gtk_widget_init_template (GTK_WIDGET (self));

    gtk_image_set_from_resource (GTK_IMAGE (priv->image), PACKAGE_RESOURCES "/icons/default-feed-cover.svg");
}

/*
 * gpod_library_item_class_init:
 * @klass: An instance of #GpodLibraryItemClass
 *
 * Initializes the various virtural functions in #GpodLibraryItemClass. In
 * addtition it connects the various #GtkWidgets from the gresource file using
 * gtk_widget_class_set_template_from_resource().
 */
static void
gpod_library_item_class_init (GpodLibraryItemClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

    object_class->set_property = gpod_library_item_set_property;
    object_class->get_property = gpod_library_item_get_property;
    object_class->finalize = gpod_library_item_finalize;

    gtk_widget_class_set_template_from_resource (widget_class,
                                                 PACKAGE_RESOURCES "/ui/widgets/library-item.ui");

    gtk_widget_class_bind_template_child_private (widget_class, GpodLibraryItem, box);
    gtk_widget_class_bind_template_child_private (widget_class, GpodLibraryItem, image);
    gtk_widget_class_bind_template_child_private (widget_class, GpodLibraryItem, title);

    properties[PROP_URL] =
        g_param_spec_string ("url",
                             "URL",
                             "The feed url",
                             NULL,
                             (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));

    properties[PROP_TITLE] =
        g_param_spec_string ("title",
                             "Title",
                             "The feed title",
                             NULL,
                             (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));

    properties[PROP_IMAGE] =
        g_param_spec_string ("image",
                             "Image",
                             "the feed image",
                             NULL,
                             (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));

    g_object_class_install_properties (object_class, LAST_PROP, properties);
}

/**
 * gpod_library_item_new:
 *
 * Creates a new #GpodLibraryItem, a custom widget to show all feeds user is
 * subscribed to.
 *
 * ## Usage
 * |[
 * GpodLibraryItem *library_item;
 *
 * library_item = gpod_library_item_new ();
 * ]|
 *
 * Returns: #GpodLibraryItem
 */
GpodLibraryItem *
gpod_library_item_new ()
{
    return g_object_new (GPOD_TYPE_LIBRARY_ITEM,
                         NULL);
}
